@@ -1,35 +1,7 @@
-"""
-
-.. autoclass:: DataLoader
-
-.. rubric:: Methods
-   
-.. autosummary::
-
-    DataLoader.decode
-    DataLoader.encode
-    DataLoader.generator
-    DataLoader.load
-    DataLoader.load_X
-    DataLoader.load_Y
-    DataLoader.load_feature_selection
-    DataLoader.load_train_test
-    DataLoader.load_train_test_identifiers
-
-.. rubric:: Attributes
-
-.. autosummary::
-
-    DataLoader.decoder
-    DataLoader.encoder
-    DataLoader.features
-    DataLoader.identifiers
-
-"""
+from   genolearn import working_directory
 import os
 import json
 import scipy.sparse
-import pandas as pd
 import numpy  as np
 import gzip
 
@@ -39,23 +11,42 @@ class DataLoader():
 
     Parameters
     ----------
-        preprocess_dir : str
-            Path to directory of preprocessed sequence data.
-
-        meta_file : str
+        meta_file   : str
             Preprocessed metadata within ``preprocess_dir``/meta
-    """
-    def __init__(self, preprocess_dir, meta_file):
+        working_dir : str
+            Path to working directory
 
-        self.dense          = os.path.exists(os.path.join(preprocess_dir, 'dense'))
-        self.preprocess_dir = preprocess_dir
-        self.data_dir       = os.path.join(preprocess_dir, 'dense' if self.dense else 'sparse')
+    """
+    def __init__(self, meta_file, working_dir = working_directory):
         
-        with open(os.path.join(preprocess_dir, 'meta', meta_file)) as f:
+        if working_dir is None:
+            raise Exception('execute GenoLearn\'s setup command first')
+
+        if not os.path.exists(working_dir) or '.genolearn' not in os.listdir(working_dir):
+            raise Exception(f'"{working_dir}" not a valid working directory')
+
+        path  = os.path.join(working_dir, 'meta')
+        if not os.path.exists(path):
+            raise Exception('no preprocessed metadata to load')
+        
+        valid = os.listdir(path)
+
+        if len(valid) == 0:
+            raise Exception(f'no preprocessed metadata in {path}')
+            
+        if meta_file not in valid:
+            raise Exception(f'"{meta_file}" not found in "{path}" - expect one of' + '\n  •'.join([''] + valid))
+
+        self.dense          = os.path.exists(os.path.join(working_dir, 'preprocess', 'dense'))
+        self.working_dir    = working_dir
+        self.preprocess_dir = os.path.join(working_dir, 'preprocess')
+        self.data_dir       = os.path.join(self.preprocess_dir, 'dense' if self.dense else 'sparse')
+        
+        with open(os.path.join(working_dir, 'meta', meta_file)) as f:
             self.meta = json.load(f)
             self.c    = len(set(self.meta['targets']))
 
-        with open(os.path.join(preprocess_dir, 'info.json')) as f:
+        with open(os.path.join(self.preprocess_dir, 'info.json')) as f:
             info = json.load(f)
             self.n, self.m = info['n'], info['m']
 
@@ -81,10 +72,18 @@ class DataLoader():
 
     def _check_identifiers(self, identifiers):
         ret = []
+        
         for identifier in identifiers:
 
+            # unlabelled data
+            if identifier == 'unlabelled':
+                for identifier in os.listdir(self.data_dir):
+                    identifier = identifier.replace('.npz', '')
+                    if identifier not in self.meta['identifiers']:
+                        ret.append(identifier)
+                    
             # check if it is a group and append entire associated identifiers
-            if identifier in self.meta['group']:
+            elif identifier in self.meta['group']:
                 ret += self.meta['group'][identifier]
             
             # check if it is Train / Test and append entire associated identifiers
@@ -192,16 +191,12 @@ class DataLoader():
         return self._features if indices is None else [self._features[i] for i in indices]
 
     def load_feature_selection(self, file):
-        ret, = np.load(os.path.join(self.preprocess_dir, 'feature-selection', file), allow_pickle = True).values()
+        ret, = np.load(os.path.join(self.working_dir, 'feature-selection', file), allow_pickle = True).values()
         return ret
 
     def generator(self, *identifiers, features = None, force_dense = False, force_sparse = False):
         """
         Iteratively yields an x, y pair from the method ``load``.
-        Yields
-        ------
-            x : load_X(identifiers, features = features, sparse = sparse)
-            y : load_Y(identifiers)
         """
         identifiers = self._check_identifiers(identifiers)
 
