@@ -23,8 +23,13 @@ def monitor_RAM():
 def set_params(params):
     """ sets the global PARAMS value """
     global PARAMS
-    PARAMS = params
+    PARAMS = params.copy()
 
+def get_params():
+    """ gets the global PARAMS value """
+    global PARAMS
+    return PARAMS.copy()
+    
 def create_log(filename, path = '.'):
     """ creates a text log file with total time elapsed and RAM usage """
     global DATETIME, START, RAM, RAMSTART, PARAMS
@@ -36,6 +41,9 @@ def create_log(filename, path = '.'):
     contents = f'datetime : {DATETIME}\nduration : {h:02.0f}h {m:02.0f}m {s:02.0f}s\nmemory   : {ram:.3f} GB'
 
     if PARAMS:
+        for key, value in PARAMS.items():
+            if isinstance(value, str):
+                PARAMS[key] = value.replace(os.path.expanduser('~'), '~')
         contents = f'{contents}\n\n{json.dumps(PARAMS, indent = 4)}'
 
     contents = f'{contents}\n'
@@ -65,9 +73,11 @@ def subdir(path, sub, ext = 0):
         return subdir(path, sub, ext + 1)
     return _sub
 
-def _gen_prompt(msg, type, default ,space, default_option):
+def _gen_prompt(msg, type, default ,space, default_option, multiple, show_choices):
+    if multiple:
+        msg = f'{msg}*'
     msg = f'{msg}' + ' ' * space
-    if isinstance(type, click.Choice):
+    if isinstance(type, click.Choice) and show_choices:
         msg = f'{msg} ' + '{' + ', '.join(map(str, type.choices)) + '}'
     if default_option:
        msg = f'{msg} [{default}]' 
@@ -91,7 +101,7 @@ def _convert_range(string):
     string = ','.join(ls)
     return string
 
-def _prompt(msg, type, default, default_option = True, multiple = False):
+def _prompt(msg, type, default, default_option = True, multiple = False, show_choices = False):
 
     # user input
     value = input(msg + ': ')
@@ -102,7 +112,7 @@ def _prompt(msg, type, default, default_option = True, multiple = False):
             value = str(default)
         else:
             print('user input required!')
-            return _prompt(msg, type, default, default_option, multiple)
+            return _prompt(msg, type, default, default_option, multiple, show_choices)
 
     # check range
     if 'range' in value:
@@ -110,11 +120,14 @@ def _prompt(msg, type, default, default_option = True, multiple = False):
         # warn and re-input if type is not int
         if (type != click.INT) and not isinstance(type, click.IntRange):
             _warn(value, type)
-            return _prompt(msg, type, default, default_option, multiple)
+            return _prompt(msg, type, default, default_option, multiple, show_choices)
 
         # convert range string to list of integers
         value = _convert_range(value)
 
+    if value == '*':
+        value = ','.join(type.choices)
+
     # detect if multi-value input
     if ',' in value:
         values = re.split(' ?, ?', value)
@@ -124,7 +137,7 @@ def _prompt(msg, type, default, default_option = True, multiple = False):
             check = _check(value, type)
             if not check:
                 _warn(value, type)
-                return _prompt(msg, type, default, default_option, multiple)
+                return _prompt(msg, type, default, default_option, multiple, show_choices)
         values = [None if value == None else value for value in map(type, values)]
         return values
 
@@ -136,7 +149,7 @@ def _prompt(msg, type, default, default_option = True, multiple = False):
             return value
         return [type(value)] if multiple else type(value)
     _warn(value, type)
-    return _prompt(msg, type, default, default_option, multiple)
+    return _prompt(msg, type, default, default_option, multiple, show_choices)
     
 def _default(info):
     if isinstance(info['type'], click.Choice) and len(info['type'].choices) == 1:
@@ -146,20 +159,21 @@ def _default(info):
 
 def prompt(params):
     prompts = []
-    hi      = 0
     for key, info in params.items():
         default = _default(info)
-        prompt = _gen_prompt(info.get('prompt', key), info['type'], default, 0, 'default' in info or default)
+        show    = info.get('show_choices', False)
+        prompt = _gen_prompt(info.get('prompt', key), info['type'], default, 0, 'default' in info or default, info.get('multiple', False), show)
         prompts.append(prompt)
-        hi     = max(hi, len(prompt))
     
+    length = max(map(len, prompts))
     config = {}
     for i, (key, info) in enumerate(params.items()):
-        space   = hi - len(prompts[i])
+        if isinstance(info['type'], click.Choice) and len(info['type'].choices) == 1:
+            config[key] = info['type'].choices[0]
+            continue
         default = _default(info)
         flag    = 'default' in info or default
-        prompt  =  _gen_prompt(info.get('prompt', key), info['type'], default, space, flag)
-        value   = _prompt(prompt, info['type'], default, flag, info.get('multiple', False))
+        value   = _prompt(f'{prompts[i]:{length}s}', info['type'], default, flag, info.get('multiple', False), info.get('show_choices', False))
         config[key] = value
     return config
 
